
import java.util.*;
import java.util.regex.*;
import java.io.*;
import java.nio.charset.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.text.html.HTML.*;

class FileReader extends InputStreamReader {
    FileReader(File file) throws IOException {
        super(new BufferedInputStream(new FileInputStream(file)));
    }
    FileReader(File file, String encoding) throws IOException {
        super(new BufferedInputStream(new FileInputStream(file)), encoding);
    }
}
class FileWriter extends OutputStreamWriter {
    FileWriter(File file) throws IOException {
        super(new BufferedOutputStream(new FileOutputStream(file)));
    }
    FileWriter(File file, String encoding) throws IOException {
        super(new BufferedOutputStream(new FileOutputStream(file)), encoding);
    }
}

public class Main {
    private final String shortName;
    private final String title;
    private final File inputDir;
    private final File outputDir;
    private final String encoding;
    private final PrintWriter help;
    private final PrintWriter index;
    private final PrintWriter toc;
    public Main(String shortName, String title, File inputDir, File outputDir, String encoding) throws Exception {
        this.shortName = shortName;
        this.title = title;
        this.inputDir = inputDir;
        this.outputDir = outputDir;
        this.encoding = encoding;
        
        outputDir.mkdirs();
        help = new PrintWriter(new FileWriter(new File(outputDir, "winhelp.hhp")));
        index = new PrintWriter(new FileWriter(new File(outputDir, "index.hhk")));
        toc = new PrintWriter(new FileWriter(new File(outputDir, "toc.hhc")));
        
        go();
        
        help.close();
        index.close();
        toc.close();
        
    }
    private void go() throws Exception {
        help.println("[OPTIONS]");
        //help.println("Compatibility=1.1 or later");
        help.println("Compiled file=" + shortName + ".chm");
        help.println("Contents file=toc.hhc");
        help.println("Default Window=Main");
        help.println("Default topic=index.html");
        //help.println("Display compile progress=No");
        help.println("Full-text search=Yes");
        help.println("Index file=index.hhk");
        //help.println("Language=0x409"); // U.S. English
        //help.println("Language=0x411"); // Japanese
        help.println("Title=" + title);
        
        help.println();
        help.println("[WINDOWS]");
        help.println("Main=\"" + toHTML(title) + "\",\"toc.hhc\",\"index.hhk\",\"index.html\",,,,,,0x63520,,0x385e,,,,,,,,0");
        
        help.println();
        help.println("[FILES]");
        //listFiles(new File("."));
        
        toc.println("<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">");
        toc.println("<HTML>");
        toc.println("<HEAD>");
        //toc.println("<meta name=\"GENERATOR\" content=\"Microsoft&reg; HTML Help Workshop 4.1\">");
        toc.println("<!-- Sitemap 1.0 -->");
        toc.println("</HEAD><BODY>");
        toc.println("<OBJECT type=\"text/site properties\">");
        toc.println("<param name=\"ImageType\" value=\"Folder\">");
        toc.println("</OBJECT>");
        
        File indexFile = new File(inputDir, "index.htm");
        if (!indexFile.exists())
            indexFile = new File(inputDir, "index.html");
        if (indexFile.exists()) {
            toc.println("<UL>");
            help.println(toHTML(indexFile.getName()));
            buildTOC(indexFile);
            toc.println("</UL>");
        }
        toc.println("</BODY></HTML>");
        
        index.println("<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">");
        index.println("<HTML>");
        index.println("<HEAD>");
        //index.println("<meta name=\"GENERATOR\" content=\"Microsoft&reg; HTML Help Workshop 4.1\">");
        index.println("<!-- Sitemap 1.0 -->");
        index.println("</HEAD><BODY>");
        index.println("<UL>");
        
        Map map = new HashMap();
        buildIndex(map, inputDir, true);
        buildIndex(map, inputDir, false);
        Iterator it4 = map.entrySet().iterator();
        while (it4.hasNext()) {
            Map.Entry entry = (Map.Entry) it4.next();
            outputIndex((String) entry.getKey(), (Collection) entry.getValue());
        }
        
        index.println("</UL>");
        index.println("</BODY></HTML>");
        
        copyFiles(inputDir, outputDir);
        
    }
    /*
    private static Set ignoredFiles = new HashSet(Arrays.asList(new String[] { "winhelp.hhp", "index.hhk", "toc.hhc" }));
    private void listFiles(File dir) {
        File[] files = dir.listFiles();
        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                if (files[i].isFile() && !ignoredFiles.contains(files[i].getName()))
                    help.println(toHTML(files[i].toString().substring(2)));
            }
            for (int i = 0; i < files.length; i++) {
                if (files[i].isDirectory())
                    listFiles(files[i]);
            }
        }
    }
    */
    
    private void buildTOC(File indexFile) throws Exception {
        if (indexFile.exists()) {
            toc.println("<LI><OBJECT type=\"text/sitemap\">");
            toc.println("  <param name=\"Name\" value=\"" + toHTML(getTitle(indexFile)) + "\">");
            toc.println("  <param name=\"Local\" value=\"" + toHTML(indexFile.toString().substring(inputDir.toString().length() + 1)) + "\">");
            toc.println("</OBJECT>");
        }
        boolean hasChild = false;
        
        File[] files = indexFile.getParentFile().listFiles();
        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                if (files[i].isDirectory()) {
                    File file = new File(files[i], "index.htm");
                    if (!file.exists())
                        file = new File(files[i], "index.html");
                    if (file.exists()) {
                        if (!hasChild) {
                            toc.println("<UL>");
                            hasChild = true;
                        }
                        buildTOC(file);
                    }
                }
            }
            if (hasChild)
                toc.println("</UL>");
        }
    }
    private String getTitle(File file) throws Exception {
        StringBuffer result = new StringBuffer();
        
        HTMLDocument doc = getHTMLDocument(file);
        Element[] elements = doc.getRootElements();
        for (int i = 0; i < elements.length; i++) {
            pickTitle(result, elements[i]);
        }
        
        return result.toString();
    }
    private void pickTitle(StringBuffer result, Element element) throws Exception {
        if (!element.isLeaf()) {
            int count = element.getElementCount();
            for (int j = 0; j < count; j++) {
                Element e = element.getElement(j);
                Tag tag = HTML.getTag(e.getName());
                if (tag == Tag.TITLE) {
                    int start = e.getStartOffset();
                    int end = e.getEndOffset();
                    String text = e.getDocument().getText(start, end - start);
                    result.append(text);
                } else {
                    pickTitle(result, e);
                }
            }
        }
    }
    
    private void buildIndex(Map map, File dir, boolean prepare) throws Exception {
        File[] files = dir.listFiles();
        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                if (files[i].isFile()) {
                    String name = files[i].getName();
                    String head = "index-";
                    String tail = ".html";
                    if (/*name.equals("index-1.html") &&*/ name.startsWith(head) && name.endsWith(tail)) {
                        System.out.println("indexing " + files[i]);
                        String mid = name.substring(head.length(), name.length() - tail.length());
                        boolean hit;
                        try {
                            Integer.parseInt(mid);
                            hit = true;
                        } catch (NumberFormatException x) {
                            hit = mid.equals("all");
                        }
                        if (hit) {
                            if (prepare) {
                                help.println(toHTML(files[i].toString().substring(inputDir.toString().length() + 1)));
                                buildIndexFromFile(map, files[i], prepare);
                            } else {
                                Map map2 = new HashMap();
                                buildIndexFromFile(map2, files[i], prepare);
                                
                                Iterator it4 = map2.entrySet().iterator();
                                while (it4.hasNext()) {
                                    Map.Entry entry = (Map.Entry) it4.next();
                                    String key = (String) entry.getKey();
                                    Collection links = (Collection) entry.getValue();
                                    Collection links2 = (Collection) map.remove(key);
                                    if (links2 != null)
                                        links.addAll(links2);
                                    outputIndex(key, links);
                                }
                            }
                        }
                    }
                }
            }
            for (int i = 0; i < files.length; i++) {
                if (files[i].isDirectory())
                    buildIndex(map, files[i], prepare);
            }
        }
    }
    private void buildIndexFromFile(Map map, File file, boolean prepare) throws Exception {
        Document doc = getHTMLDocument(file);
        
        Element[] elements = doc.getRootElements();
        for (int i = 0; i < elements.length; i++) {
            pickIndexes(map, file, elements[i], prepare);
        }
    }
    private void pickIndexes(Map map, File file, Element element, boolean prepare) throws IOException, BadLocationException {
        if (!element.isLeaf()) {
            int count = element.getElementCount();
            for (int j = 0; j < count; j++) {
                Element e = element.getElement(j);
                Tag tag = HTML.getTag(e.getName());
                if (tag == Tag.DT) {
                    String link = pickIndex(e);
                    int type = getType(link);
                    String title = null;
                    if ((type == TYPE || type == CONSTRUCTOR) && isNestedType(link)) {
                        if (prepare) {
                            title = getTitle(link);
                        }
                    } else {
                        if (!prepare) {
                            title = getTitle(link);
                        }
                    }
                    if (title != null) {
                        Collection set = (Collection) map.get(title);
                        if (set == null) {
                            set = new LinkedList();
                            map.put(title, set);
                        }
                        set.add(file.getParent().toString().replace('\\', '/') + ";" + link);
                    }
                } else {
                    pickIndexes(map, file, e, prepare);
                }
            }
        }
    }
    private String pickIndex(Element element) throws IOException, BadLocationException {
        String keyword = null;
        List items = new LinkedList();
        
        int count = element.getElementCount();
        for (int j = 0; j < count; j++) {
            Element e = element.getElement(j);
            AttributeSet attrSet = (AttributeSet) e.getAttributes().getAttribute(Tag.A);
            if (attrSet != null) {
                return (String) attrSet.getAttribute(Attribute.HREF);
            }
        }
        return null;
    }
    
    private String getTitle(String link) {
        int index = link.indexOf('#');
        if (index < 0) {
            int i = link.lastIndexOf('.');
            if (i >= 0) {
                int j = Math.max(link.lastIndexOf('.', i - 1), link.lastIndexOf('/', i - 1));
                return link.substring(j + 1, i);
            }
        } else {
            int j = link.indexOf('(', index + 1);
            if (j < 0) {
                return link.substring(index + 1);
            } else {
                int k = Math.max(index, link.lastIndexOf('.', j));
                return link.substring(k + 1, j);
            }
        }
        return null;
    }
    
    private String getInternalClassName(String link) {
        int i = link.indexOf('#');
        if (i < 0)
            i = link.length();
        i = link.lastIndexOf('.', i);   // skip .html
        
        int j = 0;
        while (j < i && link.charAt(j) == '.') {
            int k = link.indexOf('/', j);
            if (k < 0 || k + 1 >= i)
                break;
            j = k + 1;
        }
        
        return link.substring(j, i).replace('.', '$').replace('/', '.');
    }
    private String getCanonicalClassName(String link) {
        return getInternalClassName(link).replace('$', '.');
    }
    private String getClassName(String link) {
        String canonicalName = getCanonicalClassName(link);
        int index = canonicalName.lastIndexOf('.');
        return canonicalName.substring(index + 1);
    }
    
    private static final int TYPE = 0;
    private static final int CONSTRUCTOR = 1;
    private static final int METHOD = 2;
    private static final int FIELD = 3;
    private int getType(String link) {
        int index = link.indexOf('#');
        if (index < 0) {
            return TYPE;
        } else {
            int j = link.indexOf('(', index + 1);
            if (j < 0) {
                return FIELD;
            } else {
                String name = link.substring(index + 1, j);
                if (name.indexOf('.') >= 0 || name.equals(getClassName(link))) {
                    return CONSTRUCTOR;
                } else {
                    return METHOD;
                }
            }
        }
    }
    private boolean isNestedType(String link) {
        int index = link.indexOf('#');
        if (index >= 0)
            link = link.substring(0, index);
        index = link.lastIndexOf('.');
        if (index >= 0)
            link = link.substring(0, index);    // skip .html
        
        link = link.substring(link.lastIndexOf('/') + 1);
        index = link.lastIndexOf('.');
        return index >= 0;
    }
    
    private void outputIndex(String title, Collection items) {
        Set types = new TreeSet();
        Set constructors = new TreeSet();
        Map methods = new TreeMap();
        Set fields = new TreeSet();
        
        Iterator it0 = items.iterator();
        while (it0.hasNext()) {
            String link = (String) it0.next();
            int type = getType(link.substring(link.indexOf(';') + 1));
            switch (type) {
            default:
                assert false;
            case TYPE:
                types.add(link);
                break;
            case CONSTRUCTOR:
                constructors.add(link);
                break;
            case METHOD:
                {
                    String key = link.substring(link.indexOf('#') + 1);
                    String compactKey = toSimpleName(key.substring(key.indexOf('(')));
                    Map map = (Map) methods.get(compactKey);
                    if (map == null) {
                        map = new TreeMap();
                        methods.put(compactKey, map);
                    }
                    Set set = (Set) map.get(key);
                    if (set == null) {
                        set = new TreeSet();
                        map.put(key, set);
                    }
                    set.add(link);
                }
                break;
            case FIELD:
                fields.add(link);
                break;
            }
        }
        
        Iterator it1 = types.iterator();
        while (it1.hasNext()) {
            String link = (String) it1.next();
            String canonicalName = getCanonicalClassName(link.substring(link.indexOf(';') + 1));
            if (types.size() > 1) {
                outputIndex(title + " - " + canonicalName, link);
            } else {
                outputIndex(title, link);
            }
            
            if (!constructors.isEmpty()) {
                index.println("<UL>");
                Map compaction = new TreeMap();
                Iterator it = constructors.iterator();
                while (it.hasNext()) {
                    String constructor = (String) it.next();
                    if (getCanonicalClassName(constructor.substring(constructor.indexOf(';') + 1)).equals(canonicalName)) {
                        it.remove();
                        String key = constructor.substring(constructor.indexOf('#') + 1);
                        String compactKey = toSimpleName(key.substring(key.indexOf('(')));
                        Set set = (Set) compaction.get(compactKey);
                        if (set == null) {
                            set = new TreeSet();
                            compaction.put(compactKey, set);
                        }
                        set.add(constructor);
                    }
                }
                Iterator it11 = compaction.entrySet().iterator();
                while (it11.hasNext()) {
                    Map.Entry entry = (Map.Entry) it11.next();
                    String key = (String) entry.getKey();
                    Set values = (Set) entry.getValue();
                    Iterator it12 = values.iterator();
                    while (it12.hasNext()) {
                        String constructor = (String) it12.next();
                        if (values.size() > 1) {
                            key = constructor.substring(constructor.indexOf('#') + 1);
                            key = key.substring(key.indexOf('('));
                        }
                        outputIndex(key, constructor);
                    }
                }
                index.println("</UL>");
            }
        }
        
        Iterator it9 = methods.values().iterator();
        while (it9.hasNext()) {
            Map map = (Map) it9.next();
            Iterator it14 = map.entrySet().iterator();
            while (it14.hasNext()) {
                Map.Entry entry = (Map.Entry) it14.next();
                String key = (String) entry.getKey();
                Set set = (Set) entry.getValue();
                if (map.size() <= 1) {
                    key = key.substring(0, key.indexOf('(')) + toSimpleName(key.substring(key.indexOf('(')));
                    outputMethods(key, set);
                } else {
                    outputMethods(key, set);
                }
            }
        }
        
        Map compaction = new TreeMap();
        Iterator it2 = fields.iterator();
        while (it2.hasNext()) {
            String link = (String) it2.next();
            String key = getClassName(link.substring(link.indexOf(';') + 1));
            Set set = (Set) compaction.get(key);
            if (set == null) {
                set = new TreeSet();
                compaction.put(key, set);
            }
            set.add(link);
        }
        Iterator it10 = compaction.values().iterator();
        while (it10.hasNext()) {
            Set values = (Set) it10.next();
            Iterator it13 = values.iterator();
            while (it13.hasNext()) {
                String link = (String) it13.next();
                if (values.size() <= 1) {
                    outputIndex(title + " - " + getClassName(link.substring(link.indexOf(';') + 1)), link);
                } else {
                    outputIndex(title + " - " + getCanonicalClassName(link.substring(link.indexOf(';') + 1)), link);
                }
            }
        }
    }
    private String toSimpleName(String str) {
        StringTokenizer st = new StringTokenizer(str.substring(1, str.length() - 1), ",");
        StringBuffer result = new StringBuffer("(");
        while (st.hasMoreTokens()) {
            String s = st.nextToken().trim();
            s = s.substring(s.lastIndexOf('.') + 1);
            if (result.length() > 1)
                result.append(", ");
            result.append(s);
        }
        result.append(")");
        return result.toString();
    }
    private Set loadFailedClassNames = new HashSet();
    private void outputMethods(final String key, Set methodLinks) {
        final Map compaction = new TreeMap();
        Iterator it2 = methodLinks.iterator();
        while (it2.hasNext()) {
            String link = (String) it2.next();
            String compactKey = getClassName(link.substring(link.indexOf(';') + 1, link.indexOf('#')));
            compactKey = compactKey.substring(compactKey.lastIndexOf('.') + 1);
            Set set = (Set) compaction.get(compactKey);
            if (set == null) {
                set = new TreeSet();
                compaction.put(compactKey, set);
            }
            set.add(link);
        }
        
        class Method {
            String link;
            String className;
            Class clazz;
            Set childMethods = new LinkedHashSet();
            public String toString() {
                return link;
            }
        }
        final Map classNameToMethod = new TreeMap();
        final Set allMethods = new LinkedHashSet();
        Iterator it3 = methodLinks.iterator();
        while (it3.hasNext()) {
            String link = (String) it3.next();
            Method method = new Method();
            method.link = link;
            method.className = getInternalClassName(link.substring(link.indexOf(';') + 1));
            try {
                method.clazz = Class.forName(method.className);
            } catch (ClassNotFoundException x) {
                if (loadFailedClassNames.add(method.className))
                    System.err.println("warning: failde to load " + method.className + " (for " + method.link + ")");
            }
            allMethods.add(method);
            classNameToMethod.put(method.className, method);
        }
        
        Iterator it5 = allMethods.iterator();
        while (it5.hasNext()) {
            final Method method = (Method) it5.next();
            new Runnable() {
                public void run() {
                    if (method.clazz == null)
                        return;
                    
                    findSuperClasses(method, method.clazz);
                }
                public void findSuperClasses(Method method, Class clazz) {
                    Class superClass = clazz.getSuperclass();
                    Class[] interfaces = clazz.getInterfaces();
                    if (clazz.isInterface() && interfaces.length == 0) {
                        found(method, Object.class);
                    } else {
                        if (superClass != null)
                            found(method, superClass);
                        for (int i = 0; i < interfaces.length; i++) {
                            found(method, interfaces[i]);
                        }
                    }
                }
                public void found(Method method, Class superType) {
                    Method m = (Method) classNameToMethod.get(superType.getName());
                    if (m != null) {
                        m.childMethods.add(method);
                    } else {
                        findSuperClasses(method, superType);
                    }
                }
            }.run();
        }
        
        Iterator it6 = new ArrayList(allMethods).iterator();
        while (it6.hasNext()) {
            Method m = (Method) it6.next();
            allMethods.removeAll(m.childMethods);
        }
        
        final Set done = new HashSet();
        Iterator it7 = allMethods.iterator();
        while (it7.hasNext()) {
            Method m = (Method) it7.next();
            new Object() {
                public void output(Method method) {
                    if (done.add(method)) {
                        String className = method.className.replace('$', '.');
                        String simpleName = className.substring(className.lastIndexOf('.') + 1);
                        Set set = (Set) compaction.get(simpleName);
                        if (set.size() <= 1)
                            className = simpleName;
                        
                        if (allMethods.contains(method)) {
                            outputIndex(key + " - " + className, method.link);
                        } else {
                            outputIndex("- " + className, method.link);
                        }
                        if (!method.childMethods.isEmpty()) {
                            index.println("<UL>");
                            Iterator it8 = method.childMethods.iterator();
                            while (it8.hasNext()) {
                                Method m = (Method) it8.next();
                                output(m);
                            }
                            index.println("</UL>");
                        }
                    }
                }
            }.output(m);
        }
        
    }
    private void outputIndex(String title, String link) {
        index.println("<LI><OBJECT type=\"text/sitemap\">");
        index.println("  <param name=\"Name\" value=\"" + toHTML(title) + "\">");
        link = link.substring(inputDir.toString().length() + 1);
        link = link.replace(';', '/').replaceAll(" ", "");
        index.println("  <param name=\"Local\" value=\"" + toHTML(link) + "\">");
        index.println("</OBJECT>");
    }
    
    private void copyFiles(File input, File output) throws IOException {
        System.out.println("converting " + output);
        output.mkdir();
        File[] files = input.listFiles();
        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                if (!files[i].isDirectory())
                    copyFile(files[i], new File(output, files[i].getName()));
            }
            for (int i = 0; i < files.length; i++) {
                if (files[i].isDirectory())
                    copyFiles(files[i], new File(output, files[i].getName()));
            }
        }
    }
    private void copyFile(File input, File output) throws IOException {
        if (input.getName().endsWith(".html") || input.getName().endsWith(".htm")) {
            copyHTMLFile(input, output);
        } else {
            InputStream in = new FileInputStream(input);
            OutputStream out = new FileOutputStream(output);
            byte[] buffer = new byte[1024];
            for (;;) {
                int size = in.read(buffer);
                if (size < 0)
                    break;
                out.write(buffer, 0, size);
            }
            in.close();
            out.close();
        }
    }
    private void copyHTMLFile(File input, File output) throws IOException {
        StringBuffer in = new StringBuffer();
        //try {
            Reader reader = new FileReader(input, encoding);
            char[] buffer = new char[1024];
            for (;;) {
                int size = reader.read(buffer);
                if (size < 0)
                    break;
                in.append(buffer, 0, size);
            }
            reader.close();
        /*} catch (sun.io.MalformedInputException x) {
            System.err.println("error: MalformedInputException " + input);
            return;
        }*/
        
        Pattern pattern = Pattern.compile("(<A NAME=\")([^\"]*)(\">)");
        Matcher matcher = pattern.matcher(in);
        
        StringBuffer out = new StringBuffer();
        int index = 0;
        while (matcher.find()) {
            out.append(in.subSequence(index, matcher.end()));
            String name = in.substring(matcher.start(2), matcher.end(2));
            if (name.indexOf(' ') >= 0) {
                out.append(in.subSequence(matcher.start(1), matcher.end(1)));
                out.append(name.replaceAll(" ", ""));
                out.append(in.subSequence(matcher.start(3), matcher.end(3)));
            }
            index = matcher.end();
        }
        out.append(in.subSequence(index, in.length()));
        
        Writer writer = new FileWriter(output);
        writer.write(out.toString().replaceAll("<META http-equiv=\"Content-Type\" content=\"text/html; charset=[^\"]*\">", ""));
        writer.close();
    }
    
    private String toHTML(String str) {
        StringBuffer result = new StringBuffer();
        for (int i = 0; i < str.length();) {
            char ch = str.charAt(i++);
            if (ch == '&') {
                result.append("&amp;");
            } else if (ch == '<') {
                result.append("&lt;");
            } else if (ch == '>') {
                result.append("&gt;");
            } else {
                result.append(ch);
            }
        }
        return result.toString();
    }
    
    private HTMLDocument getHTMLDocument(File file) throws Exception {
        /*
        JEditorPane pane = new JEditorPane(file.toURL());
        return (HTMLDocument) pane.getDocument();
        */
        
        
        HTMLEditorKit kit = new HTMLEditorKit();
        HTMLDocument doc = (HTMLDocument) kit.createDefaultDocument();
        doc.putProperty("IgnoreCharsetDirective", Boolean.TRUE);
        Reader reader = new InputStreamReader(new BufferedInputStream(new FileInputStream(file)), encoding);
        kit.read(reader, doc, 0);
        reader.close();
        return doc;
    }
    
    private void printTree(int indent, Element element) throws IOException, BadLocationException {
        for (int i = 0; i < indent; i++)
            System.out.print("  ");
        System.out.print(element);
        int start = element.getStartOffset();
        int end = element.getEndOffset();
        System.out.print("  ");
        System.out.println(element.getDocument().getText(start, end - start));
        
        int count = element.getElementCount();
        for (int j = 0; j < count; j++) {
            Element e = element.getElement(j);
            printTree(indent + 1, e);
        }
    }
    
    
    public static void main(String[] args) {
        try {
            if (args.length != 5) {
                System.out.println("usage: html2winhelp <short name> <title> <input dir> <output dir> <input-html-encoding>");
                System.exit(2);
            }
            new Main(args[0], args[1], new File(args[2]), new File(args[3]), args[4]);
            System.exit(0);
        } catch (Exception x) {
            x.printStackTrace();
            System.exit(1);
        }
    }
    
}

